struct MultiTexVertexToPixel
{
    float4 Position         : POSITION;    
    float4 Color            : COLOR0;
    float3 Normal           : TEXCOORD0;
    float4 TextureCoords    : TEXCOORD1;
    float4 LightDirection   : TEXCOORD2;
    float4 TextureWeights   : TEXCOORD3;
    float  Depth			: TEXCOORD4;
};


struct MultiTexPixelToFrame
{
    float4 Color		: COLOR0;
};

//------- Constants --------
float4x4 View			: View;
float4x4 Projection		: Projection;
float4x4 World			: World;
float3 LightPosition	: LightDirection;
float Ambient			: Ambient;
float LightPower = 2.5;

Texture SandTexture;
sampler SandTextureSampler = sampler_state 
{ 
	texture = <SandTexture> ; 
	magfilter = LINEAR; 
	minfilter = LINEAR; 
	mipfilter=LINEAR; 
	AddressU = mirror; 
	AddressV = mirror;
};

Texture GrassTexture;
sampler GrassTextureSampler = sampler_state 
{ 
	texture = <GrassTexture> ; 
	magfilter = LINEAR; 
	minfilter = LINEAR; 
	mipfilter=LINEAR; 
	AddressU = mirror; 
	AddressV = mirror;
};

Texture RockTexture;
sampler RockTextureSampler = sampler_state 
{ 
	texture = <RockTexture> ; 
	magfilter = LINEAR; 
	minfilter = LINEAR; 
	mipfilter=LINEAR; 
	AddressU = mirror; 
	AddressV = mirror;
};

Texture SnowTexture;
sampler SnowTextureSampler = sampler_state 
{ 
	texture = <SnowTexture>; 
	magfilter = LINEAR; 
	minfilter = LINEAR; 
	mipfilter=LINEAR; 
	AddressU = mirror; 
	AddressV = mirror;
};

MultiTexVertexToPixel MultiTexturedVS( float4 inPos : POSITION, float3 inNormal: NORMAL, float4 inTexCoords: TEXCOORD0, float4 inTexWeights: TEXCOORD1)
{
    MultiTexVertexToPixel Output = (MultiTexVertexToPixel)0;
    float4x4 preViewProjection = mul (View, Projection);
    float4x4 preWorldViewProjection = mul (World, preViewProjection);
    
    Output.Position = mul(inPos, preWorldViewProjection);
    Output.Normal = mul(normalize(inNormal), World);
    Output.TextureCoords = inTexCoords;
    Output.LightDirection.xyz = LightPosition;
    Output.LightDirection.w = 1;
    Output.TextureWeights = inTexWeights;
    Output.Depth = Output.Position.z;
    
    return Output;    
}

MultiTexPixelToFrame MultiTexturedPS(MultiTexVertexToPixel PSIn)
{
    MultiTexPixelToFrame Output = (MultiTexPixelToFrame)0;
    
    float blendDistance = 30;
	float blendWidth = 50;
	float blendFactor = clamp((PSIn.Depth-blendDistance)/blendWidth, 0, 1);
	
	float4 farColor;
	farColor = tex2D(SandTextureSampler, PSIn.TextureCoords)*PSIn.TextureWeights.x;
	farColor += tex2D(GrassTextureSampler, PSIn.TextureCoords)*PSIn.TextureWeights.z;
	farColor += tex2D(RockTextureSampler, PSIn.TextureCoords)*PSIn.TextureWeights.y;
	farColor += tex2D(SnowTextureSampler, PSIn.TextureCoords)*PSIn.TextureWeights.w;

	float4 nearColor;
	float2 nearTextureCoords = PSIn.TextureCoords*3;
	nearColor = tex2D(SandTextureSampler, nearTextureCoords)*PSIn.TextureWeights.x;
	nearColor += tex2D(GrassTextureSampler, nearTextureCoords)*PSIn.TextureWeights.z;
	nearColor += tex2D(RockTextureSampler, nearTextureCoords)*PSIn.TextureWeights.y;
	nearColor += tex2D(SnowTextureSampler, nearTextureCoords)*PSIn.TextureWeights.w;
    
    float lightingFactor = saturate(saturate(dot(PSIn.Normal, normalize(PSIn.LightDirection))) + Ambient);    
   
	Output.Color = farColor*blendFactor + nearColor*(1-blendFactor);
	Output.Color *= saturate(lightingFactor + Ambient) * LightPower;

    return Output;
}

technique MultiTextured
{
    pass Pass0
    {
        VertexShader = compile vs_1_1 MultiTexturedVS();
        PixelShader = compile ps_2_0 MultiTexturedPS();
    }
}
