float4x4 View;
float4x4 Projection;
float4x4 World;
float3 LightDirection;
float Ambient;
bool EnableLighting;
float4x4 ReflectionView;
 
//------- Texture Samplers --------

Texture ReflectionMap;
sampler ReflectionSampler = sampler_state { texture = <ReflectionMap> ; magfilter = LINEAR; minfilter = LINEAR; mipfilter=LINEAR; AddressU = mirror; AddressV = mirror;};

Texture RefractionMap;
sampler RefractionSampler = sampler_state { texture = <RefractionMap> ; magfilter = LINEAR; minfilter = LINEAR; mipfilter=LINEAR; AddressU = mirror; AddressV = mirror;};


//------- Technique: Water --------
 
 struct WaterVertexToPixel
 {
     float4 Position                 : POSITION;    
     float4 ReflectionMapSamplingPos    : TEXCOORD1;
 };
 
 struct WaterPixelToFrame
 {
     float4 Color : COLOR0;
 };
 
 WaterVertexToPixel WaterVS(float4 inPos : POSITION, float2 inTex: TEXCOORD)
 {
     WaterVertexToPixel Output = (WaterVertexToPixel)0;
     float4x4 preViewProjection = mul (View, Projection);
     float4x4 preWorldViewProjection = mul (World, preViewProjection);
     float4x4 preReflectionViewProjection = mul (ReflectionView, Projection);
     float4x4 preWorldReflectionViewProjection = mul (World, preReflectionViewProjection);
     
     Output.Position = mul(inPos, preWorldViewProjection);
     Output.ReflectionMapSamplingPos = mul(inPos, preWorldReflectionViewProjection);
         
     return Output;    
 }
 
 WaterPixelToFrame WaterPS(WaterVertexToPixel PSIn)
 {
     WaterPixelToFrame Output = (WaterPixelToFrame)0;
     
     float2 ProjectedTexCoords;
     ProjectedTexCoords.x = PSIn.ReflectionMapSamplingPos.x/PSIn.ReflectionMapSamplingPos.w/2.0f + 0.5f;
     ProjectedTexCoords.y = -PSIn.ReflectionMapSamplingPos.y/PSIn.ReflectionMapSamplingPos.w/2.0f + 0.5f;
 
     Output.Color = tex2D(ReflectionSampler, ProjectedTexCoords);
 
     return Output;
 }
 
 technique Water
 {
     pass Pass0
     {
         VertexShader = compile vs_1_1 WaterVS();
         PixelShader = compile ps_2_0 WaterPS();
     }
 }
