using System;
using System.Collections.Generic;
using System.Text;
using Evolution.Engine.Interfaces;
using Microsoft.Xna.Framework.Graphics;
using Evolution.Engine.Managers;
using Microsoft.Xna.Framework;

namespace Evolution.Engine.Objects
{
    public class SkySphere : BaseObject, Evolution.Engine.Interfaces.IUpdateable, IChildRenderer, ILoadable
    {
        private Model _Model;
        private string _ModelAsset;
        private string _DomeTextureAsset;
        private Texture2D _DomeTexture;

        public SkySphere(string name, string model, string texture)
            : base(name)
        {
            _Scale = new Vector3(100, 100, 100);
            _ModelAsset = model;
            _DomeTextureAsset = texture;
        }

        #region ILoadable Members

        void ILoadable.LoadContent()
        {
            if (_Model == null)
            {
                _Model = ResourceManager.Instance.LoadContent<Model>(_ModelAsset);

                foreach (ModelMesh mesh in _Model.Meshes)
                    foreach (BasicEffect currenteffect in mesh.Effects)
                        _DomeTexture = currenteffect.Texture;

                if( _DomeTexture == null )
                    _DomeTexture = ResourceManager.Instance.LoadContent<Texture2D>(_DomeTextureAsset);

                foreach (ModelMesh modmesh in _Model.Meshes)
                    foreach (ModelMeshPart modmeshpart in modmesh.MeshParts)
                        modmeshpart.Effect = ShaderManager.Instance.GetShader(_Shader).Effect.Clone(GameEngine.Instance.Device);
            }
        }

        #endregion

        #region IChildRenderer Members

        public void RenderChildren(Microsoft.Xna.Framework.GameTime gameTime)
        {
            Effect effect = ShaderManager.Instance.GetShader(_Shader).Effect;

            foreach (ModelMesh modmesh in _Model.Meshes)
            {
                foreach (Effect currenteffect in modmesh.Effects)
                {
                    currenteffect.CurrentTechnique = effect.Techniques["Textured"];

                    currenteffect.Parameters["World"].SetValue(World);
                    currenteffect.Parameters["View"].SetValue(CameraManager.Instance.ActiveCamera.View);
                    currenteffect.Parameters["Projection"].SetValue(CameraManager.Instance.ActiveCamera.Projection);
                    currenteffect.Parameters["xTexture"].SetValue(_DomeTexture);
                }

                modmesh.Draw();
            }
        }

        #endregion

        #region IUpdateable Members

        void Evolution.Engine.Interfaces.IUpdateable.Update(Microsoft.Xna.Framework.GameTime gameTime)
        {
            _Position = PlayerManager.Instance.Player.Position;
        }

        #endregion
    }
}
